#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-08-07
# Last Modified: $Date: 2008-10-08 14:11:11 +0100 (Wed, 08 Oct 2008) $
# Id:            $Id: 10-image_graph.t 3193 2008-10-08 13:11:11Z ajb $
# $HeadURL: svn+ssh://intcvs1/repos/svn/new-pipeline-dev/useful_modules/tags/release-2.0/t/10-image_graph.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use SangerPaths qw{core ensembl badger trackingnpg tracking};
use Test::More tests => 17;
use Digest::MD5;

my $md5_object = Digest::MD5->new();

use_ok('npg_common::image::graph');

{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  my $args = {
    'width'        => 400,
    'height'       => 200,
    'title'        => 'test data',
    'x_labels_vertical' => 1,
    'x_label'      => 'Run ID',
    'y_label'      => 'PF Yield (GBases)',

  };
  my $data = [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9]];
  my $png;

  eval { $png = $graph->plotter($data, $args, q{lines}, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - with array rotate');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{f8401306fce773e2d1afd0a75f219c92}, 'md5 hex digest of png is correct');

  eval { $png = $graph->plotter($data, $args, q{lines}, undef); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - without array rotate');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{e665ff4a9e5b725061dd3db97339dc98}, 'md5 hex digest of png is correct');
}
{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  my $args = {
    'title'        => 'test data',
    'x_labels_vertical' => 1,
    'x_label'      => 'Run ID',
    'y_label'      => 'PF Yield (GBases)',
    'legend'       => ['test_1','test_2'],

  };
  my $data = [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[1,2,3,4,5,6,7,8,9,10]];
  my $png;

  eval { $png = $graph->plotter($data, $args, q{lines}, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - with legend');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{9b9b93d491d161978b50263db3639976}, 'md5 hex digest of png is correct');
}
{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  my $args = {
    'title'        => 'test data',
    'x_labels_vertical' => 1,
    'x_label'      => 'Run ID',
    'y_label'      => 'PF Yield (GBases)',
    'legend'       => ['test_1','test_2'],

  };
  my $data = [[1,2,3,4,5,6,7,8,9,10],[11,12,13,14,15,14,13,12,15,9],[1,2,3,4,5,6,7,8,9,10]];
  my $png;

  eval { $png = $graph->plotter($data); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - just data');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{f940d3b836607d660cf2d4f4708e04cb}, 'md5 hex digest of png is correct');

  eval { $png = $graph->plotter($data, undef, undef, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - just data and rotate');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{614304b6245be2939773921652ad0e2e}, 'md5 hex digest of png is correct');
}
{
  my $graph = npg_common::image::graph->new({});
  isa_ok($graph, 'npg_common::image::graph', '$graph');
  my $args = {
    'title'        => 'test data',
    'x_labels_vertical' => 1,
    'x_label'      => 'Run ID',
    'y_label'      => 'PF Yield (GBases)',
    'legend'       => ['test_1','test_2'],

  };
  my $data = [];
  my $png;

  eval { $png = $graph->plotter($data, undef, undef, 1); };
  is($EVAL_ERROR, q{}, 'no croak on generation of png graph image - no scalar@{$data}');
  $md5_object->add($png);
  is($md5_object->hexdigest(), q{14798143a22d510c6ef786a6450ee405}, 'md5 hex digest of png is correct');
}
